@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: access.cc,v 1.62 2005/11/19 19:38:45 sshwarts Exp $
+// $Id: access.cc,v 1.69 2007/07/31 20:25:52 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -26,6 +26,7 @@
 
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
+#include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
 #if BX_SUPPORT_X86_64
@@ -44,8 +45,8 @@ BX_CPU_C::write_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
   if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
     // do canonical checks
     if (!IsCanonical(offset)) {
-      BX_ERROR(("Canonical Address Failure %08x%08x",(Bit32u)(offset >> 32),(Bit32u)(offset & 0xffffffff)));
-      exception(BX_GP_EXCEPTION, 0, 0);
+      BX_ERROR(("write_virtual_checks(): canonical Failure 0x%08x:%08x", GET32H(offset), GET32L(offset)));
+      exception(int_number(seg), 0, 0);
     }
     seg->cache.valid |= SegAccessWOK;
     return;
@@ -53,14 +54,13 @@ BX_CPU_C::write_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
 #endif
   if (protected_mode()) {
     if (seg->cache.valid==0) {
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return;
+      BX_DEBUG(("write_virtual_checks(): segment descriptor not valid"));
+      exception(int_number(seg), 0, 0);
     }
 
     if (seg->cache.p == 0) { /* not present */
-      BX_INFO(("write_virtual_checks(): segment not present"));
+      BX_ERROR(("write_virtual_checks(): segment not present"));
       exception(int_number(seg), 0, 0);
-      return;
     }
 
     switch (seg->cache.type) {
@@ -70,17 +70,15 @@ BX_CPU_C::write_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
       case 10: case 11: // execute/read
       case 12: case 13: // execute only, conforming
       case 14: case 15: // execute/read-only, conforming
-        BX_INFO(("write_virtual_checks(): no write access to seg"));
+        BX_ERROR(("write_virtual_checks(): no write access to seg"));
         exception(int_number(seg), 0, 0);
-        return;
 
       case 2: case 3: /* read/write */
         if (offset > (seg->cache.u.segment.limit_scaled - length + 1)
             || (length-1 > seg->cache.u.segment.limit_scaled))
         {
-          BX_INFO(("write_virtual_checks(): write beyond limit, r/w"));
+          BX_ERROR(("write_virtual_checks(): write beyond limit, r/w"));
           exception(int_number(seg), 0, 0);
-          return;
         }
         if (seg->cache.u.segment.limit_scaled >= 7) {
           // Mark cache as being OK type for succeeding writes.  The limit
@@ -94,18 +92,16 @@ BX_CPU_C::write_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
         }
         break;
 
-      case 6: case 7: /* read write, expand down */
+      case 6: case 7: /* read/write, expand down */
         if (seg->cache.u.segment.d_b)
           upper_limit = 0xffffffff;
         else
           upper_limit = 0x0000ffff;
         if ((offset <= seg->cache.u.segment.limit_scaled) ||
-             (offset > upper_limit) ||
-             ((upper_limit - offset) < (length - 1)))
+             (offset > upper_limit) || ((upper_limit - offset) < (length - 1)))
         {
-          BX_INFO(("write_virtual_checks(): write beyond limit, r/w ED"));
+          BX_ERROR(("write_virtual_checks(): write beyond limit, r/w ED"));
           exception(int_number(seg), 0, 0);
-          return;
         }
         break;
     }
@@ -116,6 +112,7 @@ BX_CPU_C::write_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
     if (offset > (seg->cache.u.segment.limit_scaled - length + 1)
           || (length-1 > seg->cache.u.segment.limit_scaled))
     {
+      BX_DEBUG(("write_virtual_checks(): write beyond limit (real mode)"));
       exception(int_number(seg), 0, 0);
     }
     if (seg->cache.u.segment.limit_scaled >= 7) {
@@ -135,8 +132,8 @@ BX_CPU_C::read_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
   if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64) {
     // do canonical checks
     if (!IsCanonical(offset)) {
-      BX_ERROR(("Canonical Address Failure %08x%08x",(Bit32u)(offset >> 32),(Bit32u)(offset & 0xffffffff)));
-      exception(BX_GP_EXCEPTION, 0, 0);
+      BX_ERROR(("read_virtual_checks(): canonical Failure 0x%08x:%08x", GET32H(offset), GET32L(offset)));
+      exception(int_number(seg), 0, 0);
     }
     seg->cache.valid |= SegAccessROK;
     return;
@@ -144,28 +141,25 @@ BX_CPU_C::read_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
 #endif
   if (protected_mode()) {
     if (seg->cache.valid==0) {
-      BX_ERROR(("seg[%s]->selector.value = %04x", 
-         BX_CPU_THIS_PTR strseg(seg), (unsigned) seg->selector.value));
-      exception(BX_GP_EXCEPTION, 0, 0);
-      return;
+      BX_DEBUG(("read_virtual_checks(): segment descriptor not valid"));
+      exception(int_number(seg), 0, 0);
     }
 
     if (seg->cache.p == 0) { /* not present */
-      BX_INFO(("read_virtual_checks(): segment not present"));
+      BX_ERROR(("read_virtual_checks(): segment not present"));
       exception(int_number(seg), 0, 0);
-      return;
     }
 
     switch (seg->cache.type) {
       case 0: case 1: /* read only */
+      case 2: case 3: /* read/write */
       case 10: case 11: /* execute/read */
       case 14: case 15: /* execute/read-only, conforming */
         if (offset > (seg->cache.u.segment.limit_scaled - length + 1)
             || (length-1 > seg->cache.u.segment.limit_scaled))
         {
-          BX_INFO(("read_virtual_checks(): read beyond limit"));
+          BX_ERROR(("read_virtual_checks(): read beyond limit"));
           exception(int_number(seg), 0, 0);
-          return;
         }
         if (seg->cache.u.segment.limit_scaled >= 7) {
           // Mark cache as being OK type for succeeding reads. See notes for
@@ -174,23 +168,8 @@ BX_CPU_C::read_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
         }
         break;
 
-      case 2: case 3: /* read/write */
-        if (offset > (seg->cache.u.segment.limit_scaled - length + 1)
-            || (length-1 > seg->cache.u.segment.limit_scaled)) 
-        {
-          BX_INFO(("read_virtual_checks(): read beyond limit"));
-          exception(int_number(seg), 0, 0);
-          return;
-        }
-        if (seg->cache.u.segment.limit_scaled >= 7)
-        {
-          // Mark cache as being OK type for succeeding reads. See notes for
-          // write checks; similar code.
-          seg->cache.valid |= SegAccessROK;
-        }
-        break;
-
       case 4: case 5: /* read only, expand down */
+      case 6: case 7: /* read/write, expand down */
         if (seg->cache.u.segment.d_b)
           upper_limit = 0xffffffff;
         else
@@ -198,32 +177,16 @@ BX_CPU_C::read_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
         if ((offset <= seg->cache.u.segment.limit_scaled) ||
              (offset > upper_limit) || ((upper_limit - offset) < (length - 1)))
         {
-          BX_INFO(("read_virtual_checks(): read beyond limit"));
+          BX_ERROR(("read_virtual_checks(): read beyond limit"));
           exception(int_number(seg), 0, 0);
-          return;
-        }
-        break;
-
-      case 6: case 7: /* read write, expand down */
-        if (seg->cache.u.segment.d_b)
-          upper_limit = 0xffffffff;
-        else
-          upper_limit = 0x0000ffff;
-        if ((offset <= seg->cache.u.segment.limit_scaled) ||
-             (offset > upper_limit) || ((upper_limit - offset) < (length - 1)))
-        {
-          BX_INFO(("read_virtual_checks(): read beyond limit"));
-          exception(int_number(seg), 0, 0);
-          return;
         }
         break;
 
       case 8: case 9: /* execute only */
       case 12: case 13: /* execute only, conforming */
         /* can't read or write an execute-only segment */
-        BX_INFO(("read_virtual_checks(): execute only"));
+        BX_ERROR(("read_virtual_checks(): execute only"));
         exception(int_number(seg), 0, 0);
-        return;
     }
     return;
   }
@@ -231,6 +194,7 @@ BX_CPU_C::read_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
     if (offset > (seg->cache.u.segment.limit_scaled - length + 1)
         || (length-1 > seg->cache.u.segment.limit_scaled))
     {
+      BX_DEBUG(("read_virtual_checks(): read beyond limit (real mode)"));
       exception(int_number(seg), 0, 0);
     }
     if (seg->cache.u.segment.limit_scaled >= 7) {
@@ -245,13 +209,13 @@ BX_CPU_C::read_virtual_checks(bx_segment_reg_t *seg, bx_address offset,
 BX_CPU_C::strseg(bx_segment_reg_t *seg)
 {
   if (seg == &BX_CPU_THIS_PTR sregs[0]) return("ES");
-  else if (seg == & BX_CPU_THIS_PTR sregs[1]) return("CS");
-  else if (seg == & BX_CPU_THIS_PTR sregs[2]) return("SS");
+  else if (seg == &BX_CPU_THIS_PTR sregs[1]) return("CS");
+  else if (seg == &BX_CPU_THIS_PTR sregs[2]) return("SS");
   else if (seg == &BX_CPU_THIS_PTR sregs[3]) return("DS");
   else if (seg == &BX_CPU_THIS_PTR sregs[4]) return("FS");
   else if (seg == &BX_CPU_THIS_PTR sregs[5]) return("GS");
   else {
-    BX_ERROR(("undefined segment passed to strseg()!"));
+    BX_PANIC(("undefined segment passed to strseg()!"));
     return("??");
   }
 }
@@ -507,6 +471,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 2, BX_WRITE);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 1) {
+          BX_ERROR(("write_virtual_word(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit16u *hostAddr = v2h_write_word(laddr, pl);
       if (hostAddr) {
@@ -538,6 +510,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 4, BX_WRITE);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 3) {
+          BX_ERROR(("write_virtual_dword(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit32u *hostAddr = v2h_write_dword(laddr, pl);
       if (hostAddr) {
@@ -569,6 +549,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 8, BX_WRITE);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 7) {
+          BX_ERROR(("write_virtual_qword(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit64u *hostAddr = v2h_write_qword(laddr, pl);
       if (hostAddr) {
@@ -630,6 +618,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 2, BX_READ);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 1) {
+          BX_ERROR(("read_virtual_word(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit16u *hostAddr = v2h_read_word(laddr, pl);
       if (hostAddr) {
@@ -660,6 +656,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 4, BX_READ);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 3) {
+          BX_ERROR(("read_virtual_dword(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit32u *hostAddr = v2h_read_dword(laddr, pl);
       if (hostAddr) {
@@ -690,6 +694,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 8, BX_READ);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 7) {
+          BX_ERROR(("read_virtual_qword(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit64u *hostAddr = v2h_read_qword(laddr, pl);
       if (hostAddr) {
@@ -759,6 +771,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 2, BX_RW);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 1) {
+          BX_ERROR(("read_RMW_virtual_word(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit16u *hostAddr = v2h_write_word(laddr, pl);
       if (hostAddr) {
@@ -791,6 +811,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 4, BX_RW);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 3) {
+          BX_ERROR(("read_RMW_virtual_dword(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit32u *hostAddr = v2h_write_dword(laddr, pl);
       if (hostAddr) {
@@ -823,6 +851,14 @@ accessOK:
       laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
       BX_INSTR_MEM_DATA(BX_CPU_ID, laddr, 8, BX_RW);
       pl = (CPL==3);
+#if BX_CPU_LEVEL >= 4 && BX_SUPPORT_ALIGNMENT_CHECK
+      if (pl && BX_CPU_THIS_PTR alignment_check) {
+        if (laddr & 7) {
+          BX_ERROR(("read_RMW_virtual_qword(): misaligned access"));
+          exception(BX_AC_EXCEPTION, 0, 0);
+        }
+      }
+#endif
 #if BX_SupportGuest2HostTLB
       Bit64u *hostAddr = v2h_write_qword(laddr, pl);
       if (hostAddr) {
@@ -993,7 +1029,9 @@ BX_CPU_C::read_virtual_dqword(unsigned s, bx_address offset, Bit8u *data)
 BX_CPU_C::read_virtual_dqword_aligned(unsigned s, bx_address offset, Bit8u *data)
 {
   // If double quadword access is unaligned, #GP(0).
-  if (offset & 0xf) {
+  bx_address laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
+  if (laddr & 0xf) {
+    BX_DEBUG(("read_virtual_dqword_aligned: access not aligned to 16-byte"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 
@@ -1014,7 +1052,9 @@ BX_CPU_C::write_virtual_dqword(unsigned s, bx_address offset, Bit8u *data)
 BX_CPU_C::write_virtual_dqword_aligned(unsigned s, bx_address offset, Bit8u *data)
 {
   // If double quadword access is unaligned, #GP(0).
-  if (offset & 0xf) {
+  bx_address laddr = BX_CPU_THIS_PTR get_segment_base(s) + offset;
+  if (laddr & 0xf) {
+    BX_DEBUG(("write_virtual_dqword_aligned: access not aligned to 16-byte"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 
