@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: call_far.cc,v 1.12 2006/06/12 16:58:26 sshwarts Exp $
+// $Id: call_far.cc,v 1.15 2007/02/03 21:36:40 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -121,7 +121,7 @@ BX_CPU_C::call_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
     }
 
 #if BX_SUPPORT_X86_64
-    if (IsLongMode()) {
+    if (long_mode()) {
       if (gate_descriptor.type != BX_386_CALL_GATE) {
         BX_ERROR(("call_protected: gate type %u unsupported in long mode", (unsigned) gate_descriptor.type));
         exception(BX_GP_EXCEPTION, cs_raw & 0xfffc, 0);
@@ -258,7 +258,6 @@ BX_CPU_C::call_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
           unsigned room_needed, param_count;
           Bit16u   return_SS, return_CS;
           Bit32u   return_ESP, return_EIP;
-          Bit32u   return_ss_base;
           Bit16u   parameter_word[32];
           Bit32u   parameter_dword[32];
           Bit32u   temp_ESP;
@@ -348,7 +347,6 @@ BX_CPU_C::call_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
             return_ESP = ESP;
           else
             return_ESP =  SP;
-          return_ss_base = BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_SS);
 
           // save return CS:eIP to be pushed on new stack
           return_CS = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
@@ -359,14 +357,12 @@ BX_CPU_C::call_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
 
           if (gate_descriptor.type==BX_286_CALL_GATE) {
             for (unsigned i=0; i<param_count; i++) {
-              access_linear(return_ss_base + return_ESP + i*2,
-                2, 0, BX_READ, &parameter_word[i]);
+              read_virtual_word (BX_SEG_REG_SS, return_ESP + i*2, &parameter_word[i]);
             }
           }
           else {
             for (unsigned i=0; i<param_count; i++) {
-              access_linear(return_ss_base + return_ESP + i*4,
-                4, 0, BX_READ, &parameter_dword[i]);
+              read_virtual_dword(BX_SEG_REG_SS, return_ESP + i*4, &parameter_dword[i]);
             }
           }
 
@@ -530,7 +526,7 @@ BX_CPU_C::call_gate64(bx_selector_t *gate_selector)
 
     if (! IsCanonical(RSP_for_cpl_x)) {
       BX_ERROR(("call_gate64: canonical address failure %08x%08x",
-         (Bit32u)(RSP_for_cpl_x >> 32), (Bit32u)(RSP_for_cpl_x & 0xffffffff)));
+         GET32H(RSP_for_cpl_x), GET32L(RSP_for_cpl_x)));
       exception(BX_GP_EXCEPTION, 0, 0);
     }
 
@@ -565,8 +561,8 @@ BX_CPU_C::call_gate64(bx_selector_t *gate_selector)
 
     // make sure that push CS:RIP will not fail 
     if (! IsCanonical(RSP)) {
-      BX_ERROR(("call_gate64: canonical address failure %08x%08x",
-         (Bit32u)(RSP >> 32), (Bit32u)(RSP & 0xffffffff)));
+      BX_ERROR(("call_gate64: canonical address failure %08x%08x", 
+         GET32H(RSP), GET32L(RSP)));
       exception(BX_GP_EXCEPTION, 0, 0);
     }
 
