@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: ctrl_xfer_pro.cc,v 1.55 2006/06/12 16:58:26 sshwarts Exp $
+// $Id: ctrl_xfer_pro.cc,v 1.57 2007/09/10 20:47:08 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -51,7 +51,7 @@ void BX_CPU_C::check_cs(bx_descriptor_t *descriptor, Bit16u cs_raw, Bit8u check_
 #if BX_SUPPORT_X86_64
   if (descriptor->u.segment.l)
   {
-    if (! BX_CPU_THIS_PTR msr.lma) {
+    if (! BX_CPU_THIS_PTR efer.lma) {
       BX_PANIC(("check_cs: attempt to jump to long mode without enabling EFER.LMA !"));
     }
 
@@ -104,11 +104,11 @@ BX_CPU_C::load_cs(bx_selector_t *selector, bx_descriptor_t *descriptor, Bit8u cp
     (0xfffc & BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value) | cpl;
 
 #if BX_SUPPORT_X86_64
-  if (BX_CPU_THIS_PTR msr.lma) {
+  if (BX_CPU_THIS_PTR efer.lma) {
     if (descriptor->u.segment.l) {
       BX_CPU_THIS_PTR cpu_mode = BX_MODE_LONG_64;
       BX_DEBUG(("Long Mode Activated"));
-      loadSRegLMNominal(BX_SEG_REG_CS, selector->value, 0, cpl);
+      loadSRegLMNominal(BX_SEG_REG_CS, selector->value, cpl);
     }
     else {
       BX_DEBUG(("Compatibility Mode Activated"));
