@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: io.cc,v 1.36 2006/08/01 17:09:05 vruppert Exp $
+// $Id: io.cc,v 1.38 2007/07/09 15:16:12 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -39,6 +39,10 @@
 #define RAX EAX
 #endif
 
+//
+// Repeat Speedups methods
+//
+
 #if BX_SupportRepeatSpeedups
 Bit32u BX_CPU_C::FastRepINSW(bxInstruction_c *i, bx_address dstOff, Bit16u port, Bit32u wordCount)
 {
@@ -55,7 +59,7 @@ Bit32u BX_CPU_C::FastRepINSW(bxInstruction_c *i, bx_address dstOff, Bit16u port,
   write_virtual_checks(dstSegPtr, dstOff, 2);
 
   bx_address laddrDst = BX_CPU_THIS_PTR get_segment_base(BX_SEG_REG_ES) + dstOff;
-  if (BX_CPU_THIS_PTR cr0.pg)
+  if (BX_CPU_THIS_PTR cr0.get_PG())
     paddrDst = dtranslate_linear(laddrDst, CPL==3, BX_WRITE);
   else
     paddrDst = laddrDst;
@@ -174,7 +178,7 @@ Bit32u BX_CPU_C::FastRepOUTSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
   read_virtual_checks(srcSegPtr, srcOff, 2);
 
   bx_address laddrSrc = BX_CPU_THIS_PTR get_segment_base(srcSeg) + srcOff;
-  if (BX_CPU_THIS_PTR cr0.pg)
+  if (BX_CPU_THIS_PTR cr0.get_PG())
     paddrSrc = dtranslate_linear(laddrSrc, CPL==3, BX_READ);
   else
     paddrSrc = laddrSrc;
@@ -281,11 +285,34 @@ Bit32u BX_CPU_C::FastRepOUTSW(bxInstruction_c *i, unsigned srcSeg, bx_address sr
 
 #endif
 
+//
+// REP INS methods
+//
+
+void BX_CPU_C::REP_INSB_YbDX(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSB_YbDX);
+}
+
+void BX_CPU_C::REP_INSW_YwDX(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSW_YwDX);
+}
+
+void BX_CPU_C::REP_INSD_YdDX(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::INSD_YdDX);
+}
+
+//
+// INSB/INSW/INSD methods
+//
+
 void BX_CPU_C::INSB_YbDX(bxInstruction_c *i)
 {
   Bit8u value8=0;
 
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
     if (! BX_CPU_THIS_PTR allow_io(DX, 1)) {
       BX_DEBUG(("INSB_YbDX: I/O access not allowed !"));
       exception(BX_GP_EXCEPTION, 0, 0);
@@ -449,7 +476,7 @@ doIncr:
 // input doubleword from port to string
 void BX_CPU_C::INSD_YdDX(bxInstruction_c *i)
 {
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
     if (! BX_CPU_THIS_PTR allow_io(DX, 4)) {
       BX_DEBUG(("INSD_YdDX: I/O access not allowed !"));
       exception(BX_GP_EXCEPTION, 0, 0);
@@ -502,12 +529,35 @@ void BX_CPU_C::INSD_YdDX(bxInstruction_c *i)
   }
 }
 
+//
+// REP OUTS methods
+//
+
+void BX_CPU_C::REP_OUTSB_DXXb(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSB_DXXb);
+}
+
+void BX_CPU_C::REP_OUTSW_DXXw(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSW_DXXw);
+}
+
+void BX_CPU_C::REP_OUTSD_DXXd(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR repeat(i, &BX_CPU_C::OUTSD_DXXd);
+}
+
+//
+// OUTSB/OUTSW/OUTSD methods
+//
+
 void BX_CPU_C::OUTSB_DXXb(bxInstruction_c *i)
 {
   Bit8u value8;
   bx_address esi;
 
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
     if (! BX_CPU_THIS_PTR allow_io(DX, 1)) {
       BX_DEBUG(("OUTSB_DXXb: I/O access not allowed !"));
       exception(BX_GP_EXCEPTION, 0, 0);
@@ -557,7 +607,7 @@ void BX_CPU_C::OUTSW_DXXw(bxInstruction_c *i)
   bx_address esi;
   unsigned incr = 2;
 
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
     if (! BX_CPU_THIS_PTR allow_io(DX, 2)) {
       BX_DEBUG(("OUTSW_DXXw: I/O access not allowed !"));
       exception(BX_GP_EXCEPTION, 0, 0);
@@ -655,7 +705,7 @@ doIncr:
 // output doubleword string to port
 void BX_CPU_C::OUTSD_DXXd(bxInstruction_c *i)
 {
-  if (BX_CPU_THIS_PTR cr0.pe && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
+  if (BX_CPU_THIS_PTR cr0.get_PE() && (BX_CPU_THIS_PTR get_VM() || (CPL>BX_CPU_THIS_PTR get_IOPL()))) {
     if (! BX_CPU_THIS_PTR allow_io(DX, 4)) {
       BX_DEBUG(("OUTSD_DXXd: I/O access not allowed !"));
       exception(BX_GP_EXCEPTION, 0, 0);
@@ -701,6 +751,10 @@ void BX_CPU_C::OUTSD_DXXd(bxInstruction_c *i)
   }
 }
 
+//
+// non repeatable IN/OUT methods
+//
+
 void BX_CPU_C::IN_ALIb(bxInstruction_c *i)
 {
   AL = BX_CPU_THIS_PTR inp8(i->Ib());
