@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: resolve64.cc,v 1.9 2006/08/11 17:23:36 sshwarts Exp $
+// $Id: resolve64.cc,v 1.10 2007/04/09 21:15:00 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -54,9 +54,9 @@ BX_CPU_C::Resolve64Mod0Rm3(bxInstruction_c *i)
   RMAddr(i) = RBX;
 }
   void  BX_CPP_AttrRegparmN(1)
-BX_CPU_C::Resolve64Mod0Rm5(bxInstruction_c *i)
+BX_CPU_C::Resolve64Mod0Rip(bxInstruction_c *i)
 {
-  // eip hasn't been bumped yet when this is called.  must choose the saved value.
+  // RIP hasn't been bumped yet when this is called.  must choose the saved value.
   RMAddr(i) = BX_CPU_THIS_PTR prev_eip + i->ilen() + (Bit32s)i->displ32u();
 }
   void  BX_CPP_AttrRegparmN(1)
@@ -90,12 +90,6 @@ BX_CPU_C::Resolve64Mod0Rm11(bxInstruction_c *i)
   RMAddr(i) = R11;
 }
   void  BX_CPP_AttrRegparmN(1)
-BX_CPU_C::Resolve64Mod0Rm13(bxInstruction_c *i)
-{
-  // eip hasn't been bumped yet when this is called.  must choose the saved value.
-  RMAddr(i) = BX_CPU_THIS_PTR prev_eip + i->ilen() + (Bit32s)i->displ32u();
-}
-  void  BX_CPP_AttrRegparmN(1)
 BX_CPU_C::Resolve64Mod0Rm14(bxInstruction_c *i)
 {
   RMAddr(i) = R14;
