@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: dis_decode.cc,v 1.32 2006/05/12 17:04:19 sshwarts Exp $
+// $Id: dis_decode.cc,v 1.39 2007/08/18 13:51:16 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 
 #include <stdio.h>
@@ -40,11 +40,11 @@ static const unsigned char instruction_has_modrm[512] = {
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 40 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 50 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 60 */
-           1,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1, /* 0F 70 */
+           1,1,1,1,1,1,1,0,1,1,0,0,1,1,1,1, /* 0F 70 */
            0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, /* 0F 80 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F 90 */
            0,0,0,1,1,1,0,0,0,0,0,1,1,1,1,1, /* 0F A0 */
-           1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,1, /* 0F B0 */
+           1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1, /* 0F B0 */
            1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0, /* 0F C0 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F D0 */
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, /* 0F E0 */
@@ -61,6 +61,7 @@ unsigned disassembler::disasm(bx_bool is_32, bx_bool is_64, bx_address base, bx_
 
 x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_address ip, const Bit8u *instr, char *disbuf)
 {
+  if (is_64) is_32 = 1;
   x86_insn insn(is_32, is_64);
   const Bit8u *instruction_begin = instruction = instr;
   resolve_modrm = NULL;
@@ -76,6 +77,7 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
 #define SSE_PREFIX_F2   2
 #define SSE_PREFIX_F3   3      /* only one SSE prefix could be used */
   unsigned sse_prefix = SSE_PREFIX_NONE;
+  unsigned rex_prefix = 0;
 
   for(;;)
   {
@@ -100,59 +102,63 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
       case 0x4E:
       case 0x4F:
         if (! is_64) break;
-        insn.extend8b = 1;
-        if (insn.b1 & 0x8) {
-          insn.os_64 = 1;
-          insn.os_32 = 1;
-        }
-        if (insn.b1 & 0x4) insn.rex_r = 8;
-        if (insn.b1 & 0x2) insn.rex_x = 8;
-        if (insn.b1 & 0x1) insn.rex_b = 8;
+        rex_prefix = insn.b1;
         continue;
 
       case 0x26:     // ES:
         if (! is_64) insn.seg_override = ES_REG;
+        rex_prefix = 0;
         continue;
 
       case 0x2e:     // CS:
         if (! is_64) insn.seg_override = CS_REG;
+        rex_prefix = 0;
         continue;
 
       case 0x36:     // SS:
         if (! is_64) insn.seg_override = SS_REG;
+        rex_prefix = 0;
         continue;
 
       case 0x3e:     // DS:
         if (! is_64) insn.seg_override = DS_REG;
+        rex_prefix = 0;
         continue;
 
       case 0x64:     // FS:
         insn.seg_override = FS_REG;
+        rex_prefix = 0;
         continue;
 
       case 0x65:     // GS:
         insn.seg_override = GS_REG;
+        rex_prefix = 0;
         continue;
 
       case 0x66:     // operand size override
         if (!insn.os_64) insn.os_32 = !is_32;
         if (!sse_prefix) sse_prefix = SSE_PREFIX_66;
+        rex_prefix = 0;
         continue;
 
       case 0x67:     // address size override
         if (!is_64) insn.as_32 = !is_32;
         insn.as_64 = 0;
+        rex_prefix = 0;
         continue;
 
       case 0xf0:     // lock
+        rex_prefix = 0;
         continue;
 
       case 0xf2:     // repne
         if (!sse_prefix) sse_prefix = SSE_PREFIX_F2;
+        rex_prefix = 0;
         continue;
 
       case 0xf3:     // rep
         if (!sse_prefix) sse_prefix = SSE_PREFIX_F3;
+        rex_prefix = 0;
         continue;
 
       // no more prefixes
@@ -169,6 +175,17 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
     insn.b1 = 0x100 | fetch_byte();
   }
 
+  if (rex_prefix) {
+    insn.extend8b = 1;
+    if (rex_prefix & 0x8) {
+      insn.os_64 = 1;
+      insn.os_32 = 1;
+    }
+    if (rex_prefix & 0x4) insn.rex_r = 8;
+    if (rex_prefix & 0x2) insn.rex_x = 8;
+    if (rex_prefix & 0x1) insn.rex_b = 8;
+  }
+
   const BxDisasmOpcodeTable_t *opcode_table, *entry;
 
   if (is_64) {
@@ -240,6 +257,10 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
          entry = &(OPCODE_TABLE(entry)[b3 & 15]);
          break;
 
+       case _GRP64B:
+         entry = &(OPCODE_TABLE(entry)[insn.os_64]);
+         break;
+
        default:
          printf("Internal disassembler error - unknown attribute !\n");
          return x86_insn(is_32, is_64);
@@ -259,12 +280,18 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
   {
     Bit8u prefix_byte = *(instr+i);
 
-    if (prefix_byte == 0xF3 || prefix_byte == 0xF2 || prefix_byte == 0xF0) 
-    {
+    if (prefix_byte == 0xF0) {
       const BxDisasmOpcodeTable_t *prefix = &(opcode_table[prefix_byte]);
       dis_sprintf("%s ", OPCODE(prefix)->IntelOpcode);
     }
 
+    if (prefix_byte == 0xF3 || prefix_byte == 0xF2) {
+      if (attr != _GRPSSE) {
+        const BxDisasmOpcodeTable_t *prefix = &(opcode_table[prefix_byte]);
+        dis_sprintf("%s ", OPCODE(prefix)->IntelOpcode);
+      }
+    }
+
     // branch hint for jcc instructions
     if ((insn.b1 >= 0x070 && insn.b1 <= 0x07F) ||
         (insn.b1 >= 0x180 && insn.b1 <= 0x18F))
