@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: hdimage.cc,v 1.8 2006/06/16 08:56:13 vruppert Exp $
+// $Id: hdimage.cc,v 1.12 2007/09/04 07:56:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -56,6 +56,22 @@ int default_image_t::open(const char* pathname)
 
 int default_image_t::open(const char* pathname, int flags)
 {
+#ifdef WIN32
+  HANDLE hFile = CreateFile(pathname, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_RANDOM_ACCESS, NULL); 
+  if (hFile != INVALID_HANDLE_VALUE) {
+    ULARGE_INTEGER FileSize;
+    FileSize.LowPart = GetFileSize(hFile, &FileSize.HighPart);
+    CloseHandle(hFile);
+    if ((FileSize.LowPart != INVALID_FILE_SIZE) || (GetLastError() == NO_ERROR)) {
+      hd_size = FileSize.QuadPart;
+    } else {
+      return -1;
+    }
+  } else {
+    return -1;
+  }
+#endif
+
   fd = ::open(pathname, flags
 #ifdef O_BINARY
               | O_BINARY
@@ -66,16 +82,18 @@ int default_image_t::open(const char* pathname, int flags)
     return fd;
   }
 
+#ifndef WIN32
   /* look at size of image file to calculate disk geometry */
   struct stat stat_buf;
   int ret = fstat(fd, &stat_buf);
   if (ret) {
     BX_PANIC(("fstat() returns error!"));
   }
-  if ((stat_buf.st_size % 512) != 0) {
+  hd_size = (Bit64u)stat_buf.st_size;
+#endif
+  if ((hd_size % 512) != 0) {
     BX_PANIC(("size of disk image must be multiple of 512 bytes"));
   }
-  hd_size = (Bit64u)stat_buf.st_size;
 
   return fd;
 }
@@ -87,9 +105,9 @@ void default_image_t::close()
   }
 }
 
-off_t default_image_t::lseek(off_t offset, int whence)
+Bit64s default_image_t::lseek(Bit64s offset, int whence)
 {
-  return ::lseek(fd, offset, whence);
+  return (Bit64s)::lseek(fd, (off_t)offset, whence);
 }
 
 ssize_t default_image_t::read(void* buf, size_t count)
@@ -223,7 +241,7 @@ Bit64s concat_image_t::lseek(Bit64s offset, int whence)
   }
 
   seek_was_last_op = 1;
-  return ::lseek(fd, (off_t)offset, whence);
+  return (Bit64s)::lseek(fd, (off_t)offset, whence);
 }
 
 ssize_t concat_image_t::read(void* buf, size_t count)
@@ -256,9 +274,9 @@ sparse_image_t::sparse_image_t ()
   fd = -1;
   pathname = NULL;
 #ifdef _POSIX_MAPPED_FILES
- mmap_header = NULL;
+  mmap_header = NULL;
 #endif
- pagetable = NULL;
+  pagetable = NULL;
 }
 
 
@@ -461,7 +479,7 @@ void sparse_image_t::close()
  }
 }
 
-off_t sparse_image_t::lseek (off_t offset, int whence)
+Bit64s sparse_image_t::lseek(Bit64s offset, int whence)
 {
  //showpagetable(pagetable, header.numpages);
 
@@ -486,10 +504,10 @@ off_t sparse_image_t::lseek (off_t offset, int whence)
  return 0;
 }
 
-inline off_t sparse_image_t::get_physical_offset()
+inline Bit64s sparse_image_t::get_physical_offset()
 {
- off_t physical_offset = data_start;
- physical_offset += ((off_t)position_physical_page << pagesize_shift);
+ Bit64s physical_offset = data_start;
+ physical_offset += ((Bit64s)position_physical_page << pagesize_shift);
  physical_offset += position_page_offset;
 
  return physical_offset;
@@ -524,11 +542,11 @@ ssize_t sparse_image_t::read_page_fragment(Bit32u read_virtual_page, Bit32u read
  }
  else
  {
-   off_t physical_offset = get_physical_offset();
+   Bit64s physical_offset = get_physical_offset();
 
    if (physical_offset != underlying_current_filepos)
    {
-     off_t ret = ::lseek(fd, physical_offset, SEEK_SET);
+     off_t ret = ::lseek(fd, (off_t)physical_offset, SEEK_SET);
      // underlying_current_filepos update deferred
      if (ret == -1)
        panic(strerror(errno));
@@ -628,7 +646,7 @@ ssize_t sparse_image_t::write (const void* buf, size_t count)
      // We just add on another page at the end of the file
      // Reclamation, compaction etc should currently be done off-line
 
-     off_t  data_size = underlying_filesize - data_start;
+     Bit64s data_size = underlying_filesize - data_start;
      BX_ASSERT((data_size % pagesize) == 0);
 
 
@@ -638,7 +656,7 @@ ssize_t sparse_image_t::write (const void* buf, size_t count)
      pagetable[position_virtual_page] = htod32(next_data_page);
      position_physical_page = next_data_page;
 
-     off_t page_file_start = data_start + ((off_t)position_physical_page << pagesize_shift);
+     Bit64s page_file_start = data_start + ((Bit64s)position_physical_page << pagesize_shift);
 
      if (parent_image != NULL)
      {
@@ -703,11 +721,11 @@ ssize_t sparse_image_t::write (const void* buf, size_t count)
 
    BX_ASSERT(position_physical_page != SPARSE_PAGE_NOT_ALLOCATED);
 
-   off_t physical_offset = get_physical_offset();
+   Bit64s physical_offset = get_physical_offset();
 
    if (physical_offset != underlying_current_filepos)
    {
-     off_t ret = ::lseek(fd, physical_offset, SEEK_SET);
+     off_t ret = ::lseek(fd, (off_t)physical_offset, SEEK_SET);
      // underlying_current_filepos update deferred
      if (ret == -1)
        panic(strerror(errno));
@@ -828,9 +846,9 @@ void dll_image_t::close ()
    }
 }
 
-off_t dll_image_t::lseek (off_t offset, int whence)
+Bit64s dll_image_t::lseek(Bit64s offset, int whence)
 {
-      vblk = offset >> 9;
+      vblk = (int)(offset >> 9);
       return 0;
 }
 
@@ -1109,7 +1127,7 @@ Bit64s redolog_t::lseek(Bit64s offset, int whence)
     BX_PANIC( ("redolog : lseek HD with whence not SEEK_SET"));
     return -1;
   }
-  if (offset > (off_t)dtoh64(header.specific.disk))
+  if (offset > (Bit64s)dtoh64(header.specific.disk))
   {
     BX_PANIC(("redolog : lseek to byte %ld failed", (long)offset));
     return -1;
@@ -1123,143 +1141,140 @@ Bit64s redolog_t::lseek(Bit64s offset, int whence)
   return offset;
 }
 
-ssize_t
-redolog_t::read (void* buf, size_t count)
+ssize_t redolog_t::read(void* buf, size_t count)
 {
-        off_t bloc_offset, bitmap_offset;
+  Bit64s bloc_offset, bitmap_offset;
 
-        if (count != 512)
-                BX_PANIC( ("redolog : read HD with count not 512"));
+  if (count != 512)
+    BX_PANIC( ("redolog : read HD with count not 512"));
 
-        BX_DEBUG(("redolog : reading index %d, mapping to %d", extent_index, dtoh32(catalog[extent_index])));
+  BX_DEBUG(("redolog : reading index %d, mapping to %d", extent_index, dtoh32(catalog[extent_index])));
 
-        if (dtoh32(catalog[extent_index]) == REDOLOG_PAGE_NOT_ALLOCATED)
-        {
-                // page not allocated
-                return 0;
-        }
+  if (dtoh32(catalog[extent_index]) == REDOLOG_PAGE_NOT_ALLOCATED)
+  {
+    // page not allocated
+    return 0;
+  }
 
-        bitmap_offset  = (off_t)STANDARD_HEADER_SIZE + (dtoh32(header.specific.catalog) * sizeof(Bit32u));
-        bitmap_offset += (off_t)512 * dtoh32(catalog[extent_index]) * (extent_blocs + bitmap_blocs); 
-        bloc_offset    = bitmap_offset + ((off_t)512 * (bitmap_blocs + extent_offset));
+  bitmap_offset  = (Bit64s)STANDARD_HEADER_SIZE + (dtoh32(header.specific.catalog) * sizeof(Bit32u));
+  bitmap_offset += (Bit64s)512 * dtoh32(catalog[extent_index]) * (extent_blocs + bitmap_blocs);
+  bloc_offset    = bitmap_offset + ((Bit64s)512 * (bitmap_blocs + extent_offset));
 
-        BX_DEBUG(("redolog : bitmap offset is %x", (Bit32u)bitmap_offset));
-        BX_DEBUG(("redolog : bloc offset is %x", (Bit32u)bloc_offset));
+  BX_DEBUG(("redolog : bitmap offset is %x", (Bit32u)bitmap_offset));
+  BX_DEBUG(("redolog : bloc offset is %x", (Bit32u)bloc_offset));
 
+  // FIXME if same extent_index as before we can skip bitmap read
 
-        // FIXME if same extent_index as before we can skip bitmap read
+  ::lseek(fd, (off_t)bitmap_offset, SEEK_SET);
 
-        ::lseek(fd, bitmap_offset, SEEK_SET);
+  if (::read(fd, bitmap,  dtoh32(header.specific.bitmap)) != (ssize_t)dtoh32(header.specific.bitmap))
+  {
+    BX_PANIC(("redolog : failed to read bitmap for extent %d", extent_index));
+    return 0;
+  }
 
-        if (::read(fd, bitmap,  dtoh32(header.specific.bitmap)) != (ssize_t)dtoh32(header.specific.bitmap))
-        {
-                BX_PANIC(("redolog : failed to read bitmap for extent %d", extent_index));
-                return 0;
-        }
+  if ( ((bitmap[extent_offset/8] >> (extent_offset%8)) & 0x01) == 0x00 )
+  {
+    BX_DEBUG(("read not in redolog"));
 
-        if ( ((bitmap[extent_offset/8] >> (extent_offset%8)) & 0x01) == 0x00 )
-        {
-                BX_DEBUG(("read not in redolog"));
+    // bitmap says bloc not in reloglog
+    return 0;
+  }
 
-                // bitmap says bloc not in reloglog
-                return 0;
-        }
-        
-        ::lseek(fd, bloc_offset, SEEK_SET);
+  ::lseek(fd, (off_t)bloc_offset, SEEK_SET);
 
-        return (::read(fd, buf, count));
+  return (::read(fd, buf, count));
 }
 
-ssize_t
-redolog_t::write (const void* buf, size_t count)
+ssize_t redolog_t::write(const void* buf, size_t count)
 {
-        Bit32u i;
-        off_t bloc_offset, bitmap_offset, catalog_offset;
-        ssize_t written;
-        bx_bool update_catalog = 0;
+  Bit32u i;
+  Bit64s bloc_offset, bitmap_offset, catalog_offset;
+  ssize_t written;
+  bx_bool update_catalog = 0;
 
-        if (count != 512)
-                BX_PANIC( ("redolog : write HD with count not 512"));
+  if (count != 512)
+    BX_PANIC( ("redolog : write HD with count not 512"));
 
-        BX_DEBUG(("redolog : writing index %d, mapping to %d", extent_index, dtoh32(catalog[extent_index])));
-        if (dtoh32(catalog[extent_index]) == REDOLOG_PAGE_NOT_ALLOCATED)
-        {
-                if(extent_next >= dtoh32(header.specific.catalog))
-                {
-                        BX_PANIC(("redolog : can't allocate new extent... catalog is full"));
-                        return 0;
-                }
+  BX_DEBUG(("redolog : writing index %d, mapping to %d", extent_index, dtoh32(catalog[extent_index])));
+  if (dtoh32(catalog[extent_index]) == REDOLOG_PAGE_NOT_ALLOCATED)
+  {
+    if (extent_next >= dtoh32(header.specific.catalog))
+    {
+      BX_PANIC(("redolog : can't allocate new extent... catalog is full"));
+      return 0;
+    }
 
-                BX_DEBUG(("redolog : allocating new extent at %d", extent_next));
+    BX_DEBUG(("redolog : allocating new extent at %d", extent_next));
 
-                // Extent not allocated, allocate new
-                catalog[extent_index] = htod32(extent_next);
-                
-                extent_next += 1;
+    // Extent not allocated, allocate new
+    catalog[extent_index] = htod32(extent_next);
 
-                char *zerobuffer = (char*)malloc(512);
-                memset(zerobuffer, 0, 512);
+    extent_next += 1;
 
-                // Write bitmap
-                bitmap_offset  = (off_t)STANDARD_HEADER_SIZE + (dtoh32(header.specific.catalog) * sizeof(Bit32u));
-                bitmap_offset += (off_t)512 * dtoh32(catalog[extent_index]) * (extent_blocs + bitmap_blocs); 
-                ::lseek(fd, bitmap_offset, SEEK_SET);
-                for(i=0; i<bitmap_blocs; i++)
-                {
-                        ::write(fd, zerobuffer, 512);
-                }
-                // Write extent
-                for(i=0; i<extent_blocs; i++)
-                {
-                        ::write(fd, zerobuffer, 512);
-                }
+    char *zerobuffer = (char*)malloc(512);
+    memset(zerobuffer, 0, 512);
 
-                free(zerobuffer);
+    // Write bitmap
+    bitmap_offset  = (Bit64s)STANDARD_HEADER_SIZE + (dtoh32(header.specific.catalog) * sizeof(Bit32u));
+    bitmap_offset += (Bit64s)512 * dtoh32(catalog[extent_index]) * (extent_blocs + bitmap_blocs);
+    ::lseek(fd, (off_t)bitmap_offset, SEEK_SET);
+    for (i=0; i<bitmap_blocs; i++)
+    {
+      ::write(fd, zerobuffer, 512);
+    }
+    // Write extent
+    for (i=0; i<extent_blocs; i++)
+    {
+      ::write(fd, zerobuffer, 512);
+    }
 
-                update_catalog = 1;
-        }
+    free(zerobuffer);
 
-        bitmap_offset  = (off_t)STANDARD_HEADER_SIZE + (dtoh32(header.specific.catalog) * sizeof(Bit32u));
-        bitmap_offset += (off_t)512 * dtoh32(catalog[extent_index]) * (extent_blocs + bitmap_blocs); 
-        bloc_offset    = bitmap_offset + ((off_t)512 * (bitmap_blocs + extent_offset));
+    update_catalog = 1;
+  }
 
-        BX_DEBUG(("redolog : bitmap offset is %x", (Bit32u)bitmap_offset));
-        BX_DEBUG(("redolog : bloc offset is %x", (Bit32u)bloc_offset));
+  bitmap_offset  = (Bit64s)STANDARD_HEADER_SIZE + (dtoh32(header.specific.catalog) * sizeof(Bit32u));
+  bitmap_offset += (Bit64s)512 * dtoh32(catalog[extent_index]) * (extent_blocs + bitmap_blocs);
+  bloc_offset    = bitmap_offset + ((Bit64s)512 * (bitmap_blocs + extent_offset));
 
-        // Write bloc
-        ::lseek(fd, bloc_offset, SEEK_SET);
-        written = ::write(fd, buf, count);
+  BX_DEBUG(("redolog : bitmap offset is %x", (Bit32u)bitmap_offset));
+  BX_DEBUG(("redolog : bloc offset is %x", (Bit32u)bloc_offset));
 
-        // Write bitmap
-        // FIXME if same extent_index as before we can skip bitmap read
-        ::lseek(fd, bitmap_offset, SEEK_SET);
-        if (::read(fd, bitmap,  dtoh32(header.specific.bitmap)) != (ssize_t)dtoh32(header.specific.bitmap))
-        {
-                BX_PANIC(("redolog : failed to read bitmap for extent %d", extent_index));
-                return 0;
-        }
+  // Write bloc
+  ::lseek(fd, (off_t)bloc_offset, SEEK_SET);
+  written = ::write(fd, buf, count);
 
-        // If bloc does not belong to extent yet
-        if ( ((bitmap[extent_offset/8] >> (extent_offset%8)) & 0x01) == 0x00 )
-        {
-                bitmap[extent_offset/8] |= 1 << (extent_offset%8);
-                ::lseek(fd, bitmap_offset, SEEK_SET);
-                ::write(fd, bitmap,  dtoh32(header.specific.bitmap));
-        }
+  // Write bitmap
+  // FIXME if same extent_index as before we can skip bitmap read
+  ::lseek(fd, (off_t)bitmap_offset, SEEK_SET);
+  if (::read(fd, bitmap,  dtoh32(header.specific.bitmap)) != (ssize_t)dtoh32(header.specific.bitmap))
+  {
+    BX_PANIC(("redolog : failed to read bitmap for extent %d", extent_index));
+    return 0;
+  }
 
-        // Write catalog
-        if (update_catalog)
-        {
-                // FIXME if mmap
-                catalog_offset  = (off_t)STANDARD_HEADER_SIZE + (extent_index * sizeof(Bit32u));
+  // If bloc does not belong to extent yet
+  if ( ((bitmap[extent_offset/8] >> (extent_offset%8)) & 0x01) == 0x00 )
+  {
+    bitmap[extent_offset/8] |= 1 << (extent_offset%8);
+    ::lseek(fd, (off_t)bitmap_offset, SEEK_SET);
+    ::write(fd, bitmap,  dtoh32(header.specific.bitmap));
+  }
 
-                BX_DEBUG(("redolog : writing catalog at offset %x", (Bit32u)catalog_offset));
+  // Write catalog
+  if (update_catalog)
+  {
+    // FIXME if mmap
+    catalog_offset  = (Bit64s)STANDARD_HEADER_SIZE + (extent_index * sizeof(Bit32u));
 
-                ::lseek(fd, catalog_offset, SEEK_SET);
-                ::write(fd, &catalog[extent_index], sizeof(Bit32u));
-        }
+    BX_DEBUG(("redolog : writing catalog at offset %x", (Bit32u)catalog_offset));
 
-        return written;
+    ::lseek(fd, (off_t)catalog_offset, SEEK_SET);
+    ::write(fd, &catalog[extent_index], sizeof(Bit32u));
+  }
+
+  return written;
 }
 
 
@@ -1270,6 +1285,11 @@ growing_image_t::growing_image_t()
   redolog = new redolog_t();
 }
 
+growing_image_t::~growing_image_t()
+{
+  delete redolog;
+}
+
 int growing_image_t::open(const char* pathname)
 {
   int filedes = redolog->open(pathname, REDOLOG_SUBTYPE_GROWING);
@@ -1315,6 +1335,12 @@ undoable_image_t::undoable_image_t(const char* _redolog_name)
   }
 }
 
+undoable_image_t::~undoable_image_t()
+{
+  delete redolog;
+  delete ro_disk;
+}
+
 int undoable_image_t::open(const char* pathname)
 {
   char *logname=NULL;
@@ -1370,7 +1396,7 @@ void undoable_image_t::close ()
 Bit64s undoable_image_t::lseek(Bit64s offset, int whence)
 {
   redolog->lseek(offset, whence);
-  return ro_disk->lseek((off_t)offset, whence);
+  return ro_disk->lseek(offset, whence);
 }
 
 ssize_t undoable_image_t::read(void* buf, size_t count)
@@ -1403,6 +1429,12 @@ volatile_image_t::volatile_image_t(const char* _redolog_name)
   }
 }
 
+volatile_image_t::~volatile_image_t()
+{
+  delete redolog;
+  delete ro_disk;
+}
+
 int volatile_image_t::open(const char* pathname)
 {
   int filedes;
@@ -1492,10 +1524,10 @@ ssize_t volatile_image_t::write(const void* buf, size_t count)
 
 z_ro_image_t::z_ro_image_t()
 {
-  offset = (off_t)0;
+  offset = (Bit64s)0;
 }
 
-int z_ro_image_t::open (const char* pathname)
+int z_ro_image_t::open(const char* pathname)
 {
   fd = ::open(pathname, O_RDONLY
 #ifdef O_BINARY
@@ -1513,7 +1545,7 @@ int z_ro_image_t::open (const char* pathname)
   return 0;
 }
 
-void z_ro_image_t::close ()
+void z_ro_image_t::close()
 {
   if (fd > -1) {
     gzclose(gzfile);
@@ -1521,7 +1553,7 @@ void z_ro_image_t::close ()
   }
 }
 
-off_t z_ro_image_t::lseek (off_t _offset, int whence)
+Bit64s z_ro_image_t::lseek(Bit64s _offset, int whence)
 {
   // Only SEEK_SET supported
   if (whence != SEEK_SET)
@@ -1536,13 +1568,13 @@ off_t z_ro_image_t::lseek (off_t _offset, int whence)
   return offset;
 }
 
-ssize_t z_ro_image_t::read (void* buf, size_t count)
+ssize_t z_ro_image_t::read(void* buf, size_t count)
 {
   gzseek(gzfile, offset, SEEK_SET);
   return gzread(gzfile, buf, count);
 }
 
-ssize_t z_ro_image_t::write (const void* buf, size_t count)
+ssize_t z_ro_image_t::write(const void* buf, size_t count)
 {
   BX_PANIC(("z_ro_image: write not supported"));
   return 0;
@@ -1553,81 +1585,87 @@ ssize_t z_ro_image_t::write (const void* buf, size_t count)
 
 z_undoable_image_t::z_undoable_image_t(Bit64u _size, const char* _redolog_name)
 {
-        redolog = new redolog_t();
-        ro_disk = new z_ro_image_t();
-        size = _size;
+  redolog = new redolog_t();
+  ro_disk = new z_ro_image_t();
+  size = _size;
 
-        redolog_name = NULL;
-        if (_redolog_name != NULL) {
-          if (strcmp(_redolog_name,"") != 0) {
-            redolog_name = strdup(_redolog_name);
-          }
-        }
+  redolog_name = NULL;
+  if (_redolog_name != NULL) {
+    if (strcmp(_redolog_name,"") != 0) {
+      redolog_name = strdup(_redolog_name);
+    }
+  }
 }
 
-int z_undoable_image_t::open (const char* pathname)
+z_undoable_image_t::~z_undoable_image_t()
 {
-        char *logname=NULL;
-
-        if (ro_disk->open(pathname)<0)
-                return -1;
-
-        // If redolog name was set 
-        if ( redolog_name != NULL) {
-                if ( strcmp(redolog_name, "") != 0) {
-                        logname = (char*)malloc(strlen(redolog_name) + 1);
-                        strcpy (logname, redolog_name);
-                }
-        }
-
-        // Otherwise we make up the redolog filename from the pathname
-        if ( logname == NULL) {
-                logname = (char*)malloc(strlen(pathname) + UNDOABLE_REDOLOG_EXTENSION_LENGTH + 1);
-                sprintf (logname, "%s%s", pathname, UNDOABLE_REDOLOG_EXTENSION);
-        }
-
-        if (redolog->open(logname,REDOLOG_SUBTYPE_UNDOABLE,size) < 0)
-        {
-                if (redolog->create(logname, REDOLOG_SUBTYPE_UNDOABLE, size) < 0)
-                {
-                        BX_PANIC(("Can't open or create redolog '%s'",logname));
-                        return -1;
-                }
-        }
-
-        BX_INFO(("'z-undoable' disk opened, z-ro-file is '%s', redolog is '%s'", pathname, logname));
-        free(logname);
-
-        return 0;
+  delete redolog;
+  delete ro_disk;
 }
 
-void z_undoable_image_t::close ()
+int z_undoable_image_t::open(const char* pathname)
 {
-        redolog->close();
-        ro_disk->close();
+  char *logname=NULL;
 
-        if (redolog_name!=NULL)
-          free(redolog_name);
+  if (ro_disk->open(pathname)<0)
+    return -1;
+
+  // If redolog name was set 
+  if ( redolog_name != NULL) {
+    if ( strcmp(redolog_name, "") != 0) {
+      logname = (char*)malloc(strlen(redolog_name) + 1);
+      strcpy (logname, redolog_name);
+    }
+  }
+
+  // Otherwise we make up the redolog filename from the pathname
+  if ( logname == NULL) {
+    logname = (char*)malloc(strlen(pathname) + UNDOABLE_REDOLOG_EXTENSION_LENGTH + 1);
+    sprintf (logname, "%s%s", pathname, UNDOABLE_REDOLOG_EXTENSION);
+  }
+
+  if (redolog->open(logname, REDOLOG_SUBTYPE_UNDOABLE) < 0)
+  {
+    if (redolog->create(logname, REDOLOG_SUBTYPE_UNDOABLE, size) < 0)
+    {
+      BX_PANIC(("Can't open or create redolog '%s'",logname));
+      return -1;
+    }
+  }
+
+  BX_INFO(("'z-undoable' disk opened, z-ro-file is '%s', redolog is '%s'", pathname, logname));
+  free(logname);
+
+  return 0;
 }
 
-off_t z_undoable_image_t::lseek (off_t offset, int whence)
+void z_undoable_image_t::close()
 {
-      redolog->lseek(offset, whence);
-      return ro_disk->lseek(offset, whence);
+  redolog->close();
+  ro_disk->close();
+
+  if (redolog_name!=NULL)
+    free(redolog_name);
 }
 
-ssize_t z_undoable_image_t::read (void* buf, size_t count)
+Bit64s z_undoable_image_t::lseek(Bit64s offset, int whence)
 {
-      // This should be fixed if count != 512
-      if (redolog->read((char*) buf, count) != count)
-              return ro_disk->read((char*) buf, count);
-      else 
-              return count;
+  redolog->lseek(offset, whence);
+  return ro_disk->lseek(offset, whence);
 }
 
-ssize_t z_undoable_image_t::write (const void* buf, size_t count)
+ssize_t z_undoable_image_t::read(void* buf, size_t count)
 {
-      return redolog->write((char*) buf, count);
+  // This should be fixed if count != 512
+  if (redolog->read((char*) buf, count) != count)
+    return ro_disk->read((char*) buf, count);
+  else 
+    return count;
+}
+
+ssize_t z_undoable_image_t::write(const void* buf, size_t count)
+{
+  return redolog->write((char*) buf, count);
 }
 
 
@@ -1635,100 +1673,106 @@ ssize_t z_undoable_image_t::write (const void* buf, size_t count)
 
 z_volatile_image_t::z_volatile_image_t(Bit64u _size, const char* _redolog_name)
 {
-        redolog = new redolog_t();
-        ro_disk = new z_ro_image_t();
-        size = _size;
+  redolog = new redolog_t();
+  ro_disk = new z_ro_image_t();
+  size = _size;
 
-        redolog_temp = NULL;
-        redolog_name = NULL;
-        if (_redolog_name != NULL) {
-          if (strcmp(_redolog_name,"") != 0) {
-            redolog_name = strdup(_redolog_name);
-          }
-        }
+  redolog_temp = NULL;
+  redolog_name = NULL;
+  if (_redolog_name != NULL) {
+    if (strcmp(_redolog_name,"") != 0) {
+      redolog_name = strdup(_redolog_name);
+    }
+  }
 }
 
-int z_volatile_image_t::open (const char* pathname)
+z_volatile_image_t::~z_volatile_image_t()
 {
-        int filedes;
-        const char *logname=NULL;
+  delete redolog;
+  delete ro_disk;
+}
 
-        if (ro_disk->open(pathname)<0)
-                return -1;
+int z_volatile_image_t::open(const char* pathname)
+{
+  int filedes;
+  const char *logname=NULL;
 
-        // if redolog name was set 
-        if ( redolog_name != NULL) {
-                if ( strcmp(redolog_name, "") !=0 ) {
-                        logname = redolog_name;
-                }
-        }
+  if (ro_disk->open(pathname)<0)
+    return -1;
 
-        // otherwise use pathname as template
-        if (logname == NULL) {
-                logname = pathname;
-        }
+  // if redolog name was set 
+  if (redolog_name != NULL) {
+    if (strcmp(redolog_name, "") !=0 ) {
+      logname = redolog_name;
+    }
+  }
 
-        redolog_temp = (char*)malloc(strlen(logname) + VOLATILE_REDOLOG_EXTENSION_LENGTH + 1);
-        sprintf (redolog_temp, "%s%s", logname, VOLATILE_REDOLOG_EXTENSION);
+  // otherwise use pathname as template
+  if (logname == NULL) {
+    logname = pathname;
+  }
 
-        filedes = mkstemp (redolog_temp);
+  redolog_temp = (char*)malloc(strlen(logname) + VOLATILE_REDOLOG_EXTENSION_LENGTH + 1);
+  sprintf (redolog_temp, "%s%s", logname, VOLATILE_REDOLOG_EXTENSION);
+
+  filedes = mkstemp (redolog_temp);
+
+  if (filedes < 0)
+  {
+    BX_PANIC(("Can't create volatile redolog '%s'", redolog_temp));
+    return -1;
+  }
+  if (redolog->create(filedes, REDOLOG_SUBTYPE_VOLATILE, size) < 0)
+  {
+    BX_PANIC(("Can't create volatile redolog '%s'", redolog_temp));
+    return -1;
+  }
 
-        if (filedes < 0)
-        {
-                BX_PANIC(("Can't create volatile redolog '%s'", redolog_temp));
-                return -1;
-        }
-        if (redolog->create(filedes, REDOLOG_SUBTYPE_VOLATILE, size) < 0)
-        {
-                BX_PANIC(("Can't create volatile redolog '%s'", redolog_temp));
-                return -1;
-        }
-        
 #if (!defined(WIN32)) && !BX_WITH_MACOS
-        // on unix it is legal to delete an open file
-        unlink(redolog_temp);
+  // on unix it is legal to delete an open file
+  unlink(redolog_temp);
 #endif
 
-        BX_INFO(("'z-volatile' disk opened: z-ro-file is '%s', redolog is '%s'", pathname, redolog_temp));
+  BX_INFO(("'z-volatile' disk opened: z-ro-file is '%s', redolog is '%s'", pathname, redolog_temp));
 
-        return 0;
+  return 0;
 }
 
 void z_volatile_image_t::close ()
 {
-        redolog->close();
-        ro_disk->close();
+  redolog->close();
+  ro_disk->close();
 
 #if defined(WIN32) || BX_WITH_MACOS
-        // on non-unix we have to wait till the file is closed to delete it
-        unlink(redolog_temp);
+  // on non-unix we have to wait till the file is closed to delete it
+  unlink(redolog_temp);
 #endif
 
-        if (redolog_temp!=NULL)
-          free(redolog_temp);
+  if (redolog_temp!=NULL)
+    free(redolog_temp);
 
-        if (redolog_name!=NULL)
-          free(redolog_name);
+  if (redolog_name!=NULL)
+    free(redolog_name);
 }
 
-off_t z_volatile_image_t::lseek (off_t offset, int whence)
+Bit64s z_volatile_image_t::lseek(Bit64s offset, int whence)
 {
-      redolog->lseek(offset, whence);
-      return ro_disk->lseek(offset, whence);
+  redolog->lseek(offset, whence);
+  return ro_disk->lseek(offset, whence);
 }
 
 ssize_t z_volatile_image_t::read (void* buf, size_t count)
 {
-      // This should be fixed if count != 512
-      if (redolog->read((char*) buf, count) != count)
-              return ro_disk->read((char*) buf, count);
-      else 
-              return count;
+  // This should be fixed if count != 512
+  if (redolog->read((char*) buf, count) != count)
+    return ro_disk->read((char*) buf, count);
+  else 
+    return count;
 }
 
 ssize_t z_volatile_image_t::write (const void* buf, size_t count)
 {
-      return redolog->write((char*) buf, count);
+  return redolog->write((char*) buf, count);
 }
 
 #endif
