@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: serial.h,v 1.26 2005/07/10 16:51:09 vruppert Exp $
+// $Id: serial.h,v 1.31 2006/12/31 11:56:14 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2004  MandrakeSoft S.A.
@@ -25,6 +25,9 @@
 //  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 
 
+#ifndef BX_IODEV_SERIAL_H
+#define BX_IODEV_SERIAL_H
+
 // Peter Grehan (grehan@iprg.nokia.com) coded most of this
 // serial emulation.
 
@@ -40,7 +43,7 @@
 #define SERIAL_ENABLE
 #endif
 
-#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__linux__) || defined(__GNU__) || defined(__GLIBC__) || defined(__APPLE__)
+#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__linux__) || defined(__GNU__) || defined(__GLIBC__) || defined(__APPLE__) || defined(__sun__)
 #define SERIAL_ENABLE
 extern "C" {
 #include <termios.h>
@@ -199,17 +202,21 @@ typedef struct {
 
 class bx_serial_c : public bx_serial_stub_c {
 public:
-  bx_serial_c(void);
-  ~bx_serial_c(void);
-  virtual void   init(void);
-  virtual void   reset(unsigned type);
-  virtual void   serial_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state);
+  bx_serial_c();
+  virtual ~bx_serial_c();
+  virtual void init(void);
+  virtual void reset(unsigned type);
+  virtual void serial_mouse_enq(int delta_x, int delta_y, int delta_z, unsigned button_state);
+#if BX_SUPPORT_SAVE_RESTORE
+  virtual void register_state(void);
+#endif
 
 private:
   bx_serial_t s[BX_SERIAL_MAXDEV];
 
   int   detect_mouse;
   int   mouse_port;
+  int   mouse_type;
   int   mouse_delayed_dx;
   int   mouse_delayed_dy;
   int   mouse_delayed_dz;
@@ -239,5 +246,6 @@ private:
   Bit32u read(Bit32u address, unsigned io_len);
   void   write(Bit32u address, Bit32u value, unsigned io_len);
 #endif
-  };
+};
 
+#endif
